from django.test import TestCase, override_settings
from rest_framework import status
from rest_framework.test import APIClient

from django_hawk_drf.authentication import HawkAuthenticatedUser
from django_hawk_drf.tests.test_views import DjangoHawkViewTests, hawk_auth_sender


class DjangoRestFrameworkTests(DjangoHawkViewTests, TestCase):
    view_name = "test_view_set"

    @override_settings(
        DJANGO_HAWK={
            "HAWK_INCOMING_ACCESS_KEY": "some-id",
            "HAWK_INCOMING_SECRET_KEY": "some-secret",
        }
    )
    def test_user(self):
        """
        Test the user is now a HawkAuthenticatedUser
        """

        url = self.get_url()
        sender = hawk_auth_sender(url=url)
        response = APIClient().get(
            url,
            content_type="",
            HTTP_AUTHORIZATION=sender.request_header,
            HTTP_X_FORWARDED_FOR="1.2.3.4, 123.123.123.123",
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertTrue("Content-Type" in response)
        self.assertTrue("Server-Authorization" in response)
        self.assertIsInstance(response.wsgi_request.user, HawkAuthenticatedUser)
        self.assertTrue(response.wsgi_request.user.is_authenticated)
        self.assertTrue(response.wsgi_request.user.is_anonymous)



from django.urls import path

from django_hawk_drf.tests.views import ExampleViewSet

urlpatterns = [
    path(
        "test-view-set/",
        ExampleViewSet.as_view({"get": "list"}),
        name="test_view_set",
    ),
]



from typing import Optional

from django.http import HttpRequest
from django_hawk.middleware import HawkResponseMiddleware as BaseHawkResponseMiddleware
from mohawk import Receiver


class HawkResponseMiddleware(BaseHawkResponseMiddleware):
    def get_receiver(self, request: HttpRequest) -> Optional["Receiver"]:
        drf_auth = getattr(request, "auth", None)
        if isinstance(drf_auth, Receiver):
            return drf_auth
        return None



import logging
from typing import Tuple

from django.contrib.auth.models import AnonymousUser
from django_hawk.utils import DjangoHawkAuthenticationFailed, authenticate_request
from mohawk import Receiver
from rest_framework.authentication import BaseAuthentication
from rest_framework.exceptions import AuthenticationFailed
from rest_framework.request import Request

logger = logging.getLogger(__name__)


class HawkAuthenticatedUser(AnonymousUser):
    """This is a user that is authenticated using Hawk"""

    @property
    def is_authenticated(self):
        return True


class HawkAuthentication(BaseAuthentication):
    def authenticate_header(self, request) -> str:
        """
        This is returned as the WWW-Authenticate header when
        AuthenticationFailed is raised. DRF also requires this
        to send a 401 (as opposed to 403)
        """

        return "Hawk"

    def authenticate(self, request: Request) -> Tuple[HawkAuthenticatedUser, Receiver]:
        """
        Authenticates a request using Hawk signature
        If either of these suggest we cannot authenticate, AuthenticationFailed
        is raised, as required in the DRF authentication flow
        """
        try:
            hawk_receiver = authenticate_request(request=request)
        except DjangoHawkAuthenticationFailed as e:
            raise AuthenticationFailed(str(e))

        return (HawkAuthenticatedUser(), hawk_receiver)



from rest_framework.response import Response
from rest_framework.viewsets import ViewSet

from django_hawk_drf.authentication import HawkAuthentication


class ExampleViewSet(ViewSet):
    authentication_classes = (HawkAuthentication,)
    permission_classes = ()

    def list(self, request):
        return Response([])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_hawk_drf', 'django_hawk_drf.tests']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2.2,<4.2',
 'django-hawk>=1.1.0,<2.0.0',
 'djangorestframework>=3.10.3,<4.0']

setup_kwargs = {
    'name': 'django-hawk-drf',
    'version': '1.1.0',
    'description': 'Authenticate Django Rest Framework Views with django-hawk',
    'long_description': '# Django Hawk DRF\n\nThis package provides [Django Rest Framework](https://github.com/encode/django-rest-framework/) helper classes for use with [Django Hawk](https://github.com/uktrade/django-hawk).\n\n## Installation\n\nRead the [Django Hawk installation](https://github.com/uktrade/django-hawk#installation) documentation.\n\n## Example usage\n\nRead the [Django Hawk example usage](https://github.com/uktrade/django-hawk#example-usage) documentation.\n\nAdd the `HawkResponseMiddleware` to the `MIDDLEWARE` setting in your project like so:\n\n```\nMIDDLEWARE = [\n    ...\n    "django_hawk.middleware.HawkResponseMiddleware",\n    "django_hawk_drf.middleware.HawkResponseMiddleware",\n    ...\n]\n```\n\nTo check the you can use the `django_hawk.authentication.HawkAuthentication` authentication class.\n\n```python\nfrom rest_framework.response import Response\nfrom rest_framework.viewsets import ViewSet\n\nfrom django_hawk_drf.authentication import HawkAuthentication\n\n\nclass ExampleViewSet(ViewSet):\n    authentication_classes = (HawkAuthentication,)\n    permission_classes = ()\n\n    def list(self, request):\n        return Response([])\n```\n\n## Testing\n\nTests belong in the `/django_hawk_drf/tests/` directory. You can run the tests by installing the requirements like so:\n\n\n```\nmake setup\n```\n\nNow you can run the tests using the following command:\n\n```\npoetry run python manage.py test\n```\n\n### Tox tests\n\nWe use [tox](https://pypi.org/project/tox/) to test compatibility across different Django versions.\n\nTo run these tests with tox, just run the following:\n\n```\nmake tox\n```\n\n## Pushing to PyPI\n\n- [PyPI Package](https://pypi.org/project/django-hawk-drf/)\n- [Test PyPI Package](https://test.pypi.org/project/django-hawk-drf/)\n\nRunning `make build-package` will build the package into the `dist/` directory\nRunning `make push-pypi-test` will push the built package to Test PyPI\nRunning `make push-pypi` will push the built package to PyPI\n',
    'author': 'Cameron Lamb',
    'author_email': 'live.services@digital.trade.gov.uk',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/uktrade/django-hawk-drf/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0',
}


setup(**setup_kwargs)
